import * as ts from 'typescript';

export interface JsxElementNode {
  element: ts.JsxOpeningElement | ts.JsxSelfClosingElement;
  node: ts.JsxElement | ts.JsxSelfClosingElement;
}

/**
 * 查找JsxElement的节点
 * @param node
 * @param tagName
 */
export default function findJsxElement(node: ts.Node, tagName: string): JsxElementNode | undefined {
  if (node.kind !== ts.SyntaxKind.JsxElement && node.kind !== ts.SyntaxKind.JsxSelfClosingElement) {
    return;
  }

  // 找到jsx标签
  // tslint:disable-next-line:max-line-length
  const element: JsxElementNode['element'] = node.kind === ts.SyntaxKind.JsxElement ? (node as ts.JsxElement).openingElement : (node as ts.JsxSelfClosingElement);

  if (element.tagName.kind !== ts.SyntaxKind.Identifier) {
    return;
  }
  // 找到标签名
  const elementTagName = element.tagName as ts.Identifier;

  if (elementTagName.text === tagName) {
    return { element, node: node as JsxElementNode['node'] };
  }
}