import * as ts from 'typescript';
import findJsxElement from '../findJsxElement';
import getJsxElementAttr from '../getJsxElementAttr';
import sheetConfig, { Data } from '../getSheetConfig';

/** 获取组件的行为数据 */
export default function getComponentBehavior(node: ts.Node, componentTagName: string): Data | undefined {
  const element = findJsxElement(node, componentTagName);
  let data: Data | undefined;

  if (element) {
    const tagName = (element.element.tagName as ts.Identifier).text;
    const componentName = getJsxElementAttr(element.element, 'componentName');
    const sheet = sheetConfig[tagName];

    if ((sheet.requiredName && componentName) || sheet.requiredName === false) {
      data = {
        action: sheet.action,
        params: sheet.params,
        componentName: sheet.componentName
      };

      if (componentName) {
        data.componentName = (componentName as ts.Identifier).text;
      }
    }
  }

  return data;
}