import * as ts from 'typescript';
import { Data } from '../getSheetConfig';
import getLiteralValue from '../getLiteralValue';

/** 获取手动触发的行为名称 */
export default function getTriggerstatement(node: ts.Node): Data | undefined {
  let actionName: string = '';
  if (node.kind !== ts.SyntaxKind.CallExpression) {
    return;
  }

  const callExpression = node as ts.CallExpression;
  if (callExpression.expression.kind !== ts.SyntaxKind.PropertyAccessExpression) {
    return;
  }

  const propertyAccessExpression = callExpression.expression as ts.PropertyAccessExpression;

  if (propertyAccessExpression.name.text !== 'trigger') {
    return;
  }

  if (propertyAccessExpression.expression.kind !== ts.SyntaxKind.PropertyAccessExpression) {
    return;
  }

  if ((propertyAccessExpression.expression as ts.PropertyAccessExpression).name.text !== 'behavior') {
    return;
  }

  const callArguments = callExpression.arguments;

  if (callArguments[0]) {
    const argument = callArguments[0];

    if (argument.kind !== ts.SyntaxKind.ObjectLiteralExpression) {
      return;
    }

    const properties = (argument as ts.ObjectLiteralExpression).properties;

    properties.forEach((item) => {
      if (item.kind === ts.SyntaxKind.PropertyAssignment) {
        const propertyAssignment = item as ts.PropertyAssignment;

        if ((propertyAssignment.name as ts.Identifier).text === 'actionName') {
          actionName = getLiteralValue(propertyAssignment.initializer) || '动态变量';
        }
      }
    });
  }

  if (actionName) {
    console.log('getTriggerstatement', actionName)
    return {
      action: [actionName],
      componentName: '',
      params: []
    };
  }
}